@@ -13,7 +13,7 @@
 ##############################################################################
 """Text Widget Implementation
 
-$Id: checkbox.py 127425 2012-08-06 01:38:29Z rogerineichen $
+$Id$
 """
 __docformat__ = "reStructuredText"
 
@@ -24,14 +24,14 @@ import zope.schema.interfaces
 from zope.schema import vocabulary
 from zope.i18n import translate
 
-from z3c.form import interfaces, term
+from z3c.form import interfaces, term, util
 from z3c.form.widget import SequenceWidget, FieldWidget
 from z3c.form.browser import widget
 
 
+@zope.interface.implementer_only(interfaces.ICheckBoxWidget)
 class CheckBoxWidget(widget.HTMLInputWidget, SequenceWidget):
     """Input type checkbox widget implementation."""
-    zope.interface.implementsOnly(interfaces.ICheckBoxWidget)
 
     klass = u'checkbox-widget'
     css = u'checkbox'
@@ -40,22 +40,34 @@ class CheckBoxWidget(widget.HTMLInputWidget, SequenceWidget):
     def isChecked(self, term):
         return term.token in self.value
 
+    @property
+    def items(self):
+        if self.terms is None:
+            return ()
+        items = []
+        for count, term in enumerate(self.terms):
+            checked = self.isChecked(term)
+            id = '%s-%i' % (self.id, count)
+            if zope.schema.interfaces.ITitledTokenizedTerm.providedBy(term):
+                label = translate(term.title, context=self.request,
+                                  default=term.title)
+            else:
+                label = util.toUnicode(term.value)
+            items.append(
+                {'id': id, 'name': self.name + ':list', 'value': term.token,
+                 'label': label, 'checked': checked})
+        return items
+
     def update(self):
         """See z3c.form.interfaces.IWidget."""
         super(CheckBoxWidget, self).update()
         widget.addFieldClass(self)
-        self.items = []
-        for count, term in enumerate(self.terms):
-            checked = self.isChecked(term)
-            id = '%s-%i' % (self.id, count)
-            label = unicode(term.value) if not isinstance(term.value, str) \
-                else unicode(term.value, 'utf-8', errors='ignore')
-            if zope.schema.interfaces.ITitledTokenizedTerm.providedBy(term):
-                label = translate(term.title, context=self.request,
-                                  default=term.title)
-            self.items.append(
-                {'id':id, 'name':self.name + ':list', 'value':term.token,
-                 'label':label, 'checked':checked})
+
+    def json_data(self):
+        data = super(CheckBoxWidget, self).json_data()
+        data['options'] = list(self.items)
+        data['type'] = 'check'
+        return data
 
 
 @zope.component.adapter(zope.schema.interfaces.IField, interfaces.IFormLayer)
@@ -65,9 +77,9 @@ def CheckBoxFieldWidget(field, request):
     return FieldWidget(field, CheckBoxWidget(request))
 
 
+@zope.interface.implementer_only(interfaces.ISingleCheckBoxWidget)
 class SingleCheckBoxWidget(CheckBoxWidget):
     """Single Input type checkbox widget implementation."""
-    zope.interface.implementsOnly(interfaces.ISingleCheckBoxWidget)
 
     klass = u'single-checkbox-widget'
 
