@@ -13,11 +13,12 @@
 ##############################################################################
 """Data Converters
 
-$Id: converter.py 128434 2012-11-24 11:06:43Z adamg $
+$Id$
 """
 __docformat__ = "reStructuredText"
 import datetime
 import decimal
+import six
 import zope.i18n.format
 import zope.interface
 import zope.component
@@ -25,27 +26,50 @@ import zope.schema
 import zope.publisher.browser
 
 from z3c.form.i18n import MessageFactory as _
-from z3c.form import interfaces
+from z3c.form import interfaces, util
 
 
+@zope.interface.implementer(interfaces.IDataConverter)
 class BaseDataConverter(object):
     """A base implementation of the data converter."""
-    zope.interface.implements(interfaces.IDataConverter)
+
+    _strip_value = True  # Remove spaces at start and end of text line
 
     def __init__(self, field, widget):
         self.field = field
         self.widget = widget
 
+    def _getConverter(self, field):
+        # We rely on the default registered widget, this is probably a
+        # restriction for custom widgets. If so use your own MultiWidget and
+        # register your own converter which will get the right widget for the
+        # used value_type.
+        widget = zope.component.getMultiAdapter((field, self.widget.request),
+            interfaces.IFieldWidget)
+        if interfaces.IFormAware.providedBy(self.widget):
+            # form property required by objectwidget
+            widget.form = self.widget.form
+            zope.interface.alsoProvides(widget, interfaces.IFormAware)
+        converter = zope.component.getMultiAdapter(
+            (field, widget), interfaces.IDataConverter)
+        return converter
+
     def toWidgetValue(self, value):
         """See interfaces.IDataConverter"""
-        if value is self.field.missing_value:
+        if value == self.field.missing_value:
             return u''
-        return unicode(value)
+        return util.toUnicode(value)
 
     def toFieldValue(self, value):
         """See interfaces.IDataConverter"""
+        if self._strip_value and isinstance(value, six.string_types):
+            value = value.strip()
+
         if value == u'':
             return self.field.missing_value
+
+        # this is going to burp with `Object is of wrong type.`
+        # if a non-unicode value comes in from the request
         return self.field.fromUnicode(value)
 
     def __repr__(self):
@@ -67,7 +91,7 @@ class FieldDataConverter(BaseDataConverter):
             if field.__name__:
                 fieldName = '``%s`` ' % field.__name__
             raise TypeError(
-                'Field %sof type ``%s`` must provide ``IFromUnicode``.' %(
+                'Field %s of type ``%s`` must provide ``IFromUnicode``.' %(
                     fieldName, type(field).__name__))
 
 
@@ -104,7 +128,7 @@ class NumberDataConverter(BaseDataConverter):
 
     def toWidgetValue(self, value):
         """See interfaces.IDataConverter"""
-        if value is self.field.missing_value:
+        if value == self.field.missing_value:
             return u''
         return self.formatter.format(value)
 
@@ -162,7 +186,7 @@ class CalendarDataConverter(BaseDataConverter):
             return self.field.missing_value
         try:
             return self.formatter.parse(value)
-        except zope.i18n.format.DateTimeParseError, err:
+        except zope.i18n.format.DateTimeParseError as err:
             raise FormatterValidationError(err.args[0], value)
 
 
@@ -221,7 +245,7 @@ class FileUploadDataConverter(BaseDataConverter):
 
     def toWidgetValue(self, value):
         """See interfaces.IDataConverter"""
-        return value
+        return None
 
     def toFieldValue(self, value):
         """See interfaces.IDataConverter"""
@@ -241,7 +265,7 @@ class FileUploadDataConverter(BaseDataConverter):
             try:
                 seek = value.seek
                 read = value.read
-            except AttributeError, e:
+            except AttributeError as e:
                 raise ValueError(_('Bytes data are not a file object'), e)
             else:
                 seek(0)
@@ -251,7 +275,7 @@ class FileUploadDataConverter(BaseDataConverter):
                 else:
                     return self.field.missing_value
         else:
-            return str(value)
+            return util.toBytes(value)
 
 
 class SequenceDataConverter(BaseDataConverter):
@@ -262,12 +286,11 @@ class SequenceDataConverter(BaseDataConverter):
 
     def toWidgetValue(self, value):
         """Convert from Python bool to HTML representation."""
-        widget = self.widget
         # if the value is the missing value, then an empty list is produced.
         if value is self.field.missing_value:
             return []
         # Look up the term in the terms
-        terms = widget.updateTerms()
+        terms = self.widget.updateTerms()
         try:
             return [terms.getTerm(value).token]
         except LookupError:
@@ -327,7 +350,7 @@ class TextLinesConverter(BaseDataConverter):
         # if the value is the missing value, then an empty list is produced.
         if value is self.field.missing_value:
             return u''
-        return u'\n'.join(unicode(v) for v in value)
+        return u'\n'.join(util.toUnicode(v) for v in value)
 
     def toFieldValue(self, value):
         """See interfaces.IDataConverter"""
@@ -347,7 +370,7 @@ class TextLinesConverter(BaseDataConverter):
         for v in value.split('\n'):
             try:
                 items.append(valueType(v))
-            except ValueError, err:
+            except ValueError as err:
                 raise FormatterValidationError(str(err), v)
         return collectionType(items)
 
@@ -362,19 +385,7 @@ class MultiConverter(BaseDataConverter):
         """Just dispatch it."""
         if value is self.field.missing_value:
             return []
-        # We rely on the default registered widget, this is probably a
-        # restriction for custom widgets. If so use your own MultiWidget and
-        # register your own converter which will get the right widget for the
-        # used value_type.
-        field = self.field.value_type
-        widget = zope.component.getMultiAdapter((field, self.widget.request),
-            interfaces.IFieldWidget)
-        if interfaces.IFormAware.providedBy(self.widget):
-            # form property required by objectwidget
-            widget.form = self.widget.form
-            zope.interface.alsoProvides(widget, interfaces.IFormAware)
-        converter = zope.component.getMultiAdapter((field, widget),
-            interfaces.IDataConverter)
+        converter = self._getConverter(self.field.value_type)
 
         # we always return a list of values for the widget
         return [converter.toWidgetValue(v) for v in value]
@@ -384,23 +395,47 @@ class MultiConverter(BaseDataConverter):
         if not len(value):
             return self.field.missing_value
 
-        field = self.field.value_type
-        widget = zope.component.getMultiAdapter((field, self.widget.request),
-            interfaces.IFieldWidget)
-        if interfaces.IFormAware.providedBy(self.widget):
-            #form property required by objecwidget
-            widget.form = self.widget.form
-            zope.interface.alsoProvides(widget, interfaces.IFormAware)
-        converter = zope.component.getMultiAdapter((field, widget),
-            interfaces.IDataConverter)
-
+        converter = self._getConverter(self.field.value_type)
         values = [converter.toFieldValue(v) for v in value]
 
         # convert the field values to a tuple or list
         collectionType = self.field._type
+        if isinstance(collectionType, tuple):
+            collectionType = collectionType[-1]
         return collectionType(values)
 
 
+class DictMultiConverter(BaseDataConverter):
+    """Data converter for IMultiWidget."""
+
+    zope.component.adapts(
+        zope.schema.interfaces.IDict, interfaces.IMultiWidget)
+
+    def toWidgetValue(self, value):
+        """Just dispatch it."""
+        if value is self.field.missing_value:
+            return []
+        converter = self._getConverter(self.field.value_type)
+        key_converter = self._getConverter(self.field.key_type)
+
+        # we always return a list of values for the widget
+        return [
+            (key_converter.toWidgetValue(k), converter.toWidgetValue(v))
+            for k, v in value.items()]
+
+    def toFieldValue(self, value):
+        """Just dispatch it."""
+        if not len(value):
+            return self.field.missing_value
+
+        converter = self._getConverter(self.field.value_type)
+        key_converter = self._getConverter(self.field.key_type)
+
+        return dict([
+            (key_converter.toFieldValue(k), converter.toFieldValue(v))
+            for k, v in value])
+
+
 class BoolSingleCheckboxDataConverter(BaseDataConverter):
     "A special converter between boolean fields and single checkbox widgets."
 
